from ..base import (
    BaseModelTestCase,
)
from ....models.basic import NotificationBasic

"""
# ========================================================================= #
# TEST BASIC NOTIFICATION ================================================= #
# ========================================================================= #
"""


class TestBasicNotification(BaseModelTestCase):
    model = NotificationBasic

    def setUp(self):
        super(TestBasicNotification, self).setUp()

    # =================================================================== #
    # BASIC TESTS ======================================================= #
    # =================================================================== #

    def test_str(self):
        self.notification = self.model.objects.create(
            recipients=self.base_email, sender=self.base_email
        )

        self.assertNotEqual(str(self.notification), None)

    def test_with_message(self):
        message = "this is a test message"
        self.notification = self.model.objects.create(
            recipients=self.base_email, sender=self.base_email, message=message
        )

        self.assertEqual(self.notification.message, message)

    # =================================================================== #
    # BASE NOTIFICATION TESTS =========================================== #
    # =================================================================== #
    #
    # NotificationBase is abstract, so we are unable to test it directly
    # it has methods and things used in all the notification models.
    # Said methods are tested here to ensure they're covered and not
    # 'doubled up on' in all the tests
    #
    #
    # RECIPIENT
    #
    def test_recipients_empty_str(self):
        recipients = ""
        self.notification = self.model.objects.create(
            recipients=recipients, sender=self.base_email
        )
        self.assertEqual(self.notification.recipients, "")
        self.assertEqual(self.notification.recipients, "")

    def test_recipients_empty_list(self):
        recipients = []
        self.notification = self.model.objects.create(
            recipients=recipients, sender=self.base_email
        )
        self.assertEqual(self.notification.recipients, "")
        self.assertEqual(self.notification.recipients, "")

    def test_recipients_list_of_one(self):
        email1 = "danielnazarian@outlook.com"
        recipients = [email1]
        self.notification = self.model.objects.create(
            recipients=recipients, sender=self.base_email
        )
        self.assertTrue(email1 in self.notification.recipients)

    def test_recipients_list_str_of_one(self):
        email1 = "danielnazarian@outlook.com"
        recipients = str([email1])
        self.notification = self.model.objects.create(
            recipients=recipients, sender=self.base_email
        )
        self.assertTrue(email1 in self.notification.recipients)

    def test_recipients_list_of_three(self):
        email1 = "danielnazarian@outlook.com"
        email2 = "danielnazarian+20@outlook.com"
        email3 = "danielnazarian+30@outlook.com"
        recipients = [email1, email2, email3]
        self.notification = self.model.objects.create(
            recipients=recipients, sender=self.base_email
        )
        self.assertTrue(email1 in self.notification.recipients)
        self.assertTrue(email2 in self.notification.recipients)
        self.assertTrue(email3 in self.notification.recipients)

    def test_recipients_list_str_of_three(self):
        email1 = "danielnazarian@outlook.com"
        email2 = "danielnazarian+20@outlook.com"
        email3 = "danielnazarian+30@outlook.com"
        recipients = str([email1, email2, email3])
        self.notification = self.model.objects.create(
            recipients=recipients, sender=self.base_email
        )
        self.assertTrue(email1 in self.notification.recipients)
        self.assertTrue(email2 in self.notification.recipients)
        self.assertTrue(email3 in self.notification.recipients)

    #
    # RECIPIENTS CONTAINS
    #
    def test_recipients_contains_empty_str(self):
        email1 = "danielnazarian@outlook.com"
        recipients = ""
        self.notification = self.model.objects.create(
            recipients=recipients, sender=self.base_email
        )
        self.assertEqual(self.notification.recipients, "")
        self.assertEqual(self.notification.recipients_contains(email1), False)

    def test_recipients_contains_empty_list(self):
        email1 = "danielnazarian@outlook.com"
        recipients = []
        self.notification = self.model.objects.create(
            recipients=recipients, sender=self.base_email
        )
        self.assertEqual(self.notification.recipients, "")
        self.assertEqual(self.notification.recipients_contains(email1), False)

    def test_recipients_contains_different_email(self):
        email1 = "danielnazarian@outlook.com"
        recipients = ["different@email.com"]
        self.notification = self.model.objects.create(
            recipients=recipients, sender=self.base_email
        )
        self.assertEqual(self.notification.recipients, recipients[0])
        self.assertEqual(self.notification.recipients_contains(email1), False)

    def test_recipients_contains_email(self):
        email1 = "danielnazarian@outlook.com"
        recipients = [email1]
        self.notification = self.model.objects.create(
            recipients=recipients, sender=self.base_email
        )
        self.assertEqual(self.notification.recipients, email1)
        self.assertEqual(self.notification.recipients_contains(email1), True)

    def test_recipients_contains_email_when_many(self):
        email1 = "danielnazarian@outlook.com"
        recipients = [
            "danielnazarian+20@outlook.com",
            email1,
            "danielnazarian+342345@outlook.com",
        ]
        self.notification = self.model.objects.create(
            recipients=recipients, sender=self.base_email
        )
        self.assertEqual(self.notification.recipients_contains(email1), True)

    def test_recipients_does_not_contain_email_when_many(self):
        email1 = "danielnazarian@outlook.com"
        recipients = [
            "danielnazarian+20@outlook.com",
            "danielnazarian+3253@outlook.com",
            "danielnazarian+342345@outlook.com",
        ]
        self.notification = self.model.objects.create(
            recipients=recipients, sender=self.base_email
        )
        self.assertEqual(self.notification.recipients_contains(email1), False)

    #
    # RECIPIENTS CLEANUP
    #
    def test_recipients_cleanup_empty_str(self):
        recipients = ""
        self.notification = self.model.objects.create(
            recipients=recipients, sender=self.base_email
        )
        self.assertEqual(self.notification.recipients_cleanup(), "")

    def test_recipients_cleanup_one_email_str(self):
        email1 = "danielnazarian@outlook.com"
        recipients = email1
        self.notification = self.model.objects.create(
            recipients=recipients, sender=self.base_email
        )
        self.assertEqual(self.notification.recipients_cleanup(), email1)

    def test_recipients_cleanup_one_email_list(self):
        email1 = "danielnazarian@outlook.com"
        recipients = [email1]
        self.notification = self.model.objects.create(
            recipients=recipients, sender=self.base_email
        )
        self.assertEqual(self.notification.recipients_cleanup(), email1)

    def test_recipients_cleanup_one_email_list_str(self):
        email1 = "danielnazarian@outlook.com"
        recipients = str([email1])
        self.notification = self.model.objects.create(
            recipients=recipients, sender=self.base_email
        )
        self.assertEqual(self.notification.recipients_cleanup(), email1)

    def test_recipients_cleanup_many_email_str(self):
        email1 = "danielnazarian@outlook.com"
        email2 = "danielnazarian+3523@outlook.com"
        email3 = "danielnazarian+436546@outlook.com"
        recipients = f"{email1},{email2},{email3}"
        self.notification = self.model.objects.create(
            recipients=recipients, sender=self.base_email
        )
        res = self.notification.recipients_cleanup()
        self.assertTrue(email1 in res)
        self.assertTrue(email2 in res)
        self.assertTrue(email3 in res)
        self.assertEqual(len(res.split(",")), 3)

    def test_recipients_cleanup_many_email_list(self):
        email1 = "danielnazarian@outlook.com"
        email2 = "danielnazarian+3523@outlook.com"
        email3 = "danielnazarian+436546@outlook.com"
        recipients = [email1, email2, email3]
        self.notification = self.model.objects.create(
            recipients=recipients, sender=self.base_email
        )
        res = self.notification.recipients_cleanup()
        self.assertTrue(email1 in res)
        self.assertTrue(email2 in res)
        self.assertTrue(email3 in res)
        self.assertEqual(len(res.split(",")), 3)

    def test_recipients_cleanup_many_email_list_str(self):
        email1 = "danielnazarian@outlook.com"
        email2 = "danielnazarian+3523@outlook.com"
        email3 = "danielnazarian+436546@outlook.com"
        recipients = str([email1, email2, email3])
        self.notification = self.model.objects.create(
            recipients=recipients, sender=self.base_email
        )
        res = self.notification.recipients_cleanup()
        self.assertTrue(email1 in res)
        self.assertTrue(email2 in res)
        self.assertTrue(email3 in res)
        self.assertEqual(len(res.split(",")), 3)



from ..base import BaseModelTestCase
from ....models.email import NotificationEmail

"""
# ========================================================================= #
# TEST NOTIFICATION EMAIL MANAGER ========================================= #
# ========================================================================= #
"""


class TestNotificationEmailManager(BaseModelTestCase):
    email = "test+1@example.com"
    password = "password"

    def setUp(self):
        super(TestNotificationEmailManager, self).setUp()

    # =================================================================== #
    # BASIC TESTS ======================================================= #
    # =================================================================== #

    def test_send_email_template_doesnt_exist(self):
        try:
            notification_email = NotificationEmail.objects.send_email(
                template="INVALID",
            )
            self.assertTrue(False)
        except ValueError as e:
            self.assertTrue(True)

    def test_send_email_template_does_exist(self):
        template = "emails/default.html"
        notification_email = NotificationEmail.objects.send_email(
            template=template,
        )

        self.assertEqual(notification_email.template.path, template)

    def test_send_email_no_recipients(self):
        template = "emails/default.html"
        recipients = []
        notification_email = NotificationEmail.objects.send_email(
            template=template, recipients=recipients
        )

        self.assertEqual(notification_email.recipients, "")

    def test_send_email_with_one_recipients_list(self):
        email1 = "danielnazarian@outlook.com"
        template = "emails/default.html"
        recipients = [email1]
        notification_email = NotificationEmail.objects.send_email(
            template=template, recipients=recipients
        )

        self.assertEqual(notification_email.recipients, email1)

    def test_send_email_with_one_recipients_str(self):
        email1 = "danielnazarian@outlook.com"
        template = "emails/default.html"
        recipients = str([email1])
        notification_email = NotificationEmail.objects.send_email(
            template=template, recipients=recipients
        )

        self.assertEqual(notification_email.recipients, email1)

    def test_send_email_with_many_recipients_list(self):
        email1 = "danielnazarian@outlook.com"
        email2 = "danielnazarian+32523@outlook.com"
        email3 = "danielnazarian+352346@outlook.com"
        template = "emails/default.html"
        recipients = [email1, email2, email3]
        notification_email = NotificationEmail.objects.send_email(
            template=template, recipients=recipients
        )

        self.assertTrue(email1 in notification_email.recipients)
        self.assertTrue(email2 in notification_email.recipients)
        self.assertTrue(email3 in notification_email.recipients)

    def test_send_email_with_many_recipients_str(self):
        email1 = "danielnazarian@outlook.com"
        email2 = "danielnazarian+32523@outlook.com"
        email3 = "danielnazarian+352346@outlook.com"
        template = "emails/default.html"
        recipients = str([email1, email2, email3])
        notification_email = NotificationEmail.objects.send_email(
            template=template, recipients=recipients
        )

        self.assertTrue(email1 in notification_email.recipients)
        self.assertTrue(email2 in notification_email.recipients)
        self.assertTrue(email3 in notification_email.recipients)

    def test_send_email_with_context_dict(self):
        context = {"user": "2563468934986734986"}
        template = "emails/default.html"
        notification_email = NotificationEmail.objects.send_email(
            template=template, context=context
        )

        self.assertEqual(notification_email.context, context)



#  Created by Daniel Nazarian on 8/8/21, 1:13 PM
#  Copyright (c) Daniel Nazarian. 2021 . All rights reserved.
#
#  Please do NOT use, edit, distribute, or otherwise use this code without consent.
#  For questions, comments, concerns, and more -> dnaz@danielnazarian.com
from rest_framework.routers import DefaultRouter

from .views.basic import NotificationBasicViewSet
from .views.email import NotificationEmailViewSet
from .views.push import NotificationPushViewSet

"""
# =================================================================================================== #
# URLS ============================================================================================== #
# =================================================================================================== #
"""

# Create a router and register our view sets with it
router = DefaultRouter()
router.register(r"email", NotificationEmailViewSet)
router.register(r"basic", NotificationBasicViewSet)
router.register(r"push", NotificationPushViewSet)
urlpatterns = router.urls



from django.core.exceptions import ValidationError
from django.utils import timezone
from rest_framework import viewsets
from rest_framework.permissions import IsAuthenticated

from ..helpers import str_to_bool, api_response_error, api_response_success
from ..models.basic import NotificationBasic
from ..serializers.basic import NotificationBasicSerializer

"""
============================================================================================ #
BASIC ====================================================================================== #
============================================================================================ #
"""


# =============================== #
#
# NOTIFICATION BASIC VIEW SET
#
class NotificationBasicViewSet(viewsets.GenericViewSet):
    queryset = NotificationBasic.objects.all()
    serializer_class = NotificationBasicSerializer
    permission_classes = (IsAuthenticated,)

    def list(self, request, *args, **kwargs):
        """
        Endpoint to list out Basic Notifications
        """
        serializer_class = self.get_serializer_class()
        serializer = serializer_class(
            self.filter_queryset(
                self.queryset.filter(recipients__contains=request.user.email)
            ),
            many=True,
            context={"request": request},
        )
        page = self.paginate_queryset(serializer.data)
        return self.get_paginated_response(page)

    def retrieve(self, request, *args, **kwargs):
        """
        Endpoint to retrieve specific Basic Notification

        :param uuid pk: UUID of NotificationBasic to retrieve
        """
        pk = self.kwargs.get("pk")
        try:
            notification_basic = NotificationBasic.objects.get(pk=pk)
            if not notification_basic.recipients_contains(request.user.email):
                raise NotificationBasic.DoesNotExist
        except (NotificationBasic.DoesNotExist, ValidationError):
            return api_response_error("Notification not found.")

        # serializer and return
        serializer = self.get_serializer_class()(
            notification_basic, context={"request": request}
        )
        return api_response_success(serializer.data)

    def create(self, request, *args, **kwargs):
        """
        Endpoint to create Basic Notification
        """
        request_data_copy = request.data.copy()
        request_data_copy["sender"] = request.user.email
        request_data_copy["datetime_sent"] = timezone.now()
        request_data_copy["sent_successfully"] = True

        try:
            serializer_class = self.get_serializer_class()
            serializer = serializer_class(request_data_copy)
            NotificationBasic.objects.create(**serializer.data)
        except (ValidationError, TypeError) as e:
            return self.response_handler.response_error(
                message="Error creating notification. Please try again later.", error=e
            )

        try:
            return api_response_success(serializer.data)
        except (AttributeError,) as e:
            return api_response_error(e)

    def partial_update(self, request, *args, **kwargs):
        """
        Endpoint to update specific Basic Notification

        :param uuid pk: UUID of NotificationBasic to update

        :param (body, optional) bool read: mark NotificationBasic read or ont
        """
        pk = self.kwargs.get("pk")
        try:
            notification_basic = NotificationBasic.objects.get(pk=pk)
            if not notification_basic.recipients_contains(request.user.email):
                raise NotificationBasic.DoesNotExist
        except (NotificationBasic.DoesNotExist, ValidationError):
            return api_response_error("Notification not found.")

        # update object
        if request.data.get("read"):
            notification_basic.read = str_to_bool(request.data.get("read"))

        notification_basic.save()

        # serializer and return
        serializer = self.get_serializer_class()(
            notification_basic, context={"request": request}
        )
        return api_response_success(serializer.data)
